<?php
namespace WPFunnels\Admin\FunnelType;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;
use WPFunnels\Admin\FunnelType\Wpfnl_Funnel_Type;
use WPFunnels\Wpfnl_functions;

class Wpfunnels_Lms_Checkout  {

    public function __construct() {
        add_filter( 'wpfunnels/checkout_columns', [ $this,'change_checkout_columns_in_admin' ], 10, 2 );
        add_filter( 'wpfunnels/ob_positions', [ $this,'change_ob_positions_in_admin' ], 10, 2 );
    }



    /**
     * Change checkout column name in checkout tab 
     * 
     * @return Array
     * @since 1.0.0 
     */
    public function change_checkout_columns_in_admin( $columns, $funnel_id ){

        if( $funnel_id ){
            $type = get_post_meta( $funnel_id, '_wpfnl_funnel_type', true );
            if( 'lms' === $type ){
                $columns = [];
                $columns = [
                    'product-name' 			=> 'Course',
                    'product-price' 		=> 'Lessons',
                    'product-quantity' 		=> 'Type',
                    'total-price' 			=> 'Price',
                    'product-action' 		=> 'Actions',
                ];
            }
        }
        return $columns;
    }

    /**
     * Change order bump position for lms
     * 
     * @param Array
     * @param String
     * 
     * @return Array
     */
    public function change_ob_positions_in_admin( $positions, $funnel_id ){
        if( $funnel_id ){
            $type = get_post_meta( $funnel_id, '_wpfnl_funnel_type', true );
            if( 'lms' === $type ){
                $positions = [];
                $positions = [
                    [
                        'name'  => 'Before product preview',
                        'value' => 'before-lms-order',
                    ],
                    [
                        'name'  => 'After product preview',
                        'value' => 'after-lms-order',
                    ],
                    [
                        'name'  => 'Pop-up offer',
                        'value' => 'popup',
                    ],
                ];
            }
        }
        return $positions;
    }


    /**
     * Fetch course when search course in canvas for checkout/upsell/downsell
     * 
     * @param $term
     * @return Array
     */
    public function retrieve_item( $term = '' ){
        
        $formatted_courses = [];
        if( $term && Wpfnl_lms_learndash_functions::is_learndash_active() ){
            $courses = Wpfnl_lms_learndash_functions::search_course($term);
            if( !empty($courses) ){
                foreach( $courses as $key=>$course ){
                    $course_meta = learndash_get_course_meta_setting($course->ID);
                    if( $course_meta['sfwd-courses_course_price_type'] === 'open' || $course_meta['sfwd-courses_course_price_type'] === 'closed'){
                        continue;
                    }
                    $price    = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
                    $price	  = isset( $price['price'] ) ? $price['price'] : $price;
                    $formatted_courses[$course->ID] = [
                        'name'       => $course->post_title,
                        'price'      => $price,
                        'sale_price' => $price,
                    ];
                }
            }
        }
        return $formatted_courses;
    }


    /**
     * Save checkout item to post meta 
     * 
     * @param Array $payload
     * @param String $product_id
     * @param Array $saved_products
     * 
     * @since 2.4.6
     */
    public function save_items( $payload, $product_id, $saved_products ){
    
        if ($saved_products && !empty($saved_products) ) {
            return [
                'success' => false,
                'message' => 'Product already added',
                'products' => [],
            ];
        }
        $course   = Wpfnl_lms_learndash_functions::get_course_by_id( $product_id );
        $course_details   = Wpfnl_lms_learndash_functions::get_course_details_by_id( $product_id );
        $price    = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
        $price	  = isset( $price['price'] ) ? $price['price'] : $price;
        $image_id = get_post_meta( $product_id, '_thumbnail_id', true );
        $image    = wp_get_attachment_image_src( $image_id, 'single-post-thumbnail' );

        return [
            'success' => true,
            'products' => [
                'id' 				=> $product_id,
                'title' 			=> $course->post_title,
                'price' 			=> $price,
                'trial_price' 		=> $course_details['trial_price'],
                'numeric_price'     => $price,
                'no_of_lession'     => $course_details['no_of_lesson'],
                'type'     			=> $course_details['type'],
                'currency' 			=> function_exists( 'learndash_get_currency_symbol' ) ? Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_get_currency_symbol()) : Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_30_get_currency_symbol()),
                'sale_price' 		=> $price,
                'regular_price' 	=> $price,
                'quantity' 			=> 1,
                'image' 			=> isset($image[0]) ? $image[0] : '',
                'discount_type' 	=> '',
                'discount_value' 	=> '',
                'product_edit_link' => get_edit_post_link($product_id),
                'product_view_link' => get_permalink($product_id),
            ],
        ];
    
    }


    /**
     * Get courses for checkout step
     * 
     * @param String $step_id
     * @return Array
     */
    public function get_items( $step_id = '' ){
        $response = [];
        
        if( $step_id ){
            $products 			=  get_post_meta($step_id, '_wpfnl_checkout_products', true);
            $discount 			=  get_post_meta($step_id, '_wpfnl_checkout_discount_main_product', true);
            if( $products && Wpfnl_lms_learndash_functions::is_learndash_active()){
                $course   = Wpfnl_lms_learndash_functions::get_course_by_id( $products[ 0 ][ 'id' ] );
                $course_details   = Wpfnl_lms_learndash_functions::get_course_details_by_id( $products[ 0 ][ 'id' ] );
                $price    = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
                $price	  = isset( $price['price'] ) ? $price['price'] : $price;
                $image_id = get_post_meta( $products[ 0 ][ 'id' ], '_thumbnail_id', true );
                $image    = wp_get_attachment_image_src( $image_id, 'single-post-thumbnail' );

                $response['products'][] = array(
                    'id' 				=> $products[0]['id'],
                    'title' 			=> $course->post_title,
                    'price' 			=> $price,
                    'trial_price' 		=> $course_details['trial_price'],
                    'numeric_price'     => $price,
                    'no_of_lession'     => $course_details['no_of_lesson'],
                    'type'     			=> $course_details['type'],
                    'currency' 			=> function_exists( 'learndash_get_currency_symbol' ) ? Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_get_currency_symbol()) : Wpfnl_lms_learndash_functions::get_currency_symbol_from_country_code(learndash_30_get_currency_symbol()),
                    'sale_price' 		=> $price,
                    'regular_price' 	=> $price,
                    'quantity' 			=> 1,
                    'image' 			=> isset( $image[0] ) ? $image[0] : '',
                    'discount_type' 	=> '',
                    'discount_value' 	=> '',
                    'product_edit_link' => get_edit_post_link($products[0]['id']),
                    'product_view_link' => get_permalink($products[0]['id']),
                );
                $response['coupon'] = '';
                $response['isMultipleProduct'] = '';
                $response['isQuantity'] = '';
                $response['discount'] = $discount;
            }else{
                $response['success'] = true;
                $response['products'] = [];
                $response['coupon'] = '';
                $response['isMultipleProduct'] = '';
                $response['isQuantity'] = '';
                $response['discount'] = $discount;
            }
            $response['columns'] = Wpfnl_functions::get_checkout_columns( $step_id );
        }
        return $response;
       
    }


    /**
     * Retrieve order bump item
     */
    public function retrieve_ob_item( $term = '' ){
        $formatted_courses = [];
        if( $term && Wpfnl_lms_learndash_functions::is_learndash_active() ){
            
            $courses = Wpfnl_lms_learndash_functions::search_course($term);
			foreach( $courses as $key=>$course ){
				$course_meta = learndash_get_course_meta_setting($course->ID);
				if( $course_meta['sfwd-courses_course_price_type'] === 'open' || $course_meta['sfwd-courses_course_price_type'] === 'closed'){
					continue;
				}
				$cr_meta = Wpfnl_lms_learndash_functions::get_course_details_by_id($course->ID);
				$price    = function_exists( 'learndash_get_course_price' ) ? learndash_get_course_price( $course ) : '';
				$price	  = isset( $price['price'] ) ? $price['price'] : $price;

				$image_id = get_post_meta($course->ID,'_thumbnail_id', true);
				$image = wp_get_attachment_image_src($image_id, 'single-post-thumbnail');
				$course_meta = learndash_get_course_meta_setting($course->ID);
				$formatted_courses[$course->ID] = [
					'name'       => $course->post_title,
					'price'      => $cr_meta['price'],
					'sale_price' => '',
					'html_price' => '',
					'title' => $course->post_title,
					'img' => array(
						'id' => $image_id ? $image_id : '',
						'url' => isset($image[0]) ? $image[0] : "",
					),
					'description' => $course_meta['sfwd-courses_course_materials']
				];
			}
        }
        return $formatted_courses;
    }
    
}
