<?php
namespace WPFunnels\FunnelType;

use WPFunnels\Wpfnl_functions;
use WPFunnels\FunnelType\Wpfnl_Public_Funnel_Type;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;


class Wpfnl_Public_Lms extends Wpfnl_Public_Funnel_Type
{

    /**
     * When order bump accept/reject for lms
     * 
     * @param
     * @return Array
     * 
     */
    public function wpfnl_order_bump_trigger( $step_id, $product_id, $quantity, $key, $user_id, $funnel_id , $checker ){
        $data = [];
        if ($checker == "true") {
            $data = $this->wpfnl_order_bump_accept( $step_id, $product_id, $quantity, $key, $user_id, $funnel_id );
        }else{
            $data = $this->wpfnl_order_bump_reject( $step_id, $product_id, $quantity, $key, $user_id, $funnel_id );
        }
        return $data;
    }


    /**
     * When order bump accept for lms
     * 
     * @param
     * @return Array
     */
    private function wpfnl_order_bump_accept( $step_id, $product_id, $quantity, $key, $user_id, $funnel_id ){
        
        if( Wpfnl_lms_learndash_functions::is_learndash_active() ){
            $course = Wpfnl_lms_learndash_functions::get_course_details_by_id($product_id);
            $html = $this->render_main_product_design( $course , $step_id, $funnel_id);
            $paypal_nonce = learndash_paypal_init_user_purchase_hash( $user_id, $product_id );

            $lms_stripe_version = '';
            
            if ( defined( 'LEARNDASH_STRIPE_VERSION' ) ) {
                $lms_stripe_version = LEARNDASH_STRIPE_VERSION;
            }

            $stripe_nonce = '';
            if (version_compare($lms_stripe_version, '1.9.2', '<=')) {
                $stripe_nonce = wp_create_nonce( 'stripe-nonce-' . $product_id . $course['price']*100 . $course['type'] );
            }else{
                $stripe_nonce = wp_create_nonce( 'stripe-nonce-' . $product_id . $course['type'] );
            }
            
            update_user_meta(
                1,
                'ld_purchase_nonce_' . $paypal_nonce,
                array(
                    'user_id'    => $user_id,
                    'product_id' => $product_id,
                    'time'       => time(),
                    'nonce'=> $paypal_nonce,
                )
            );

            $data = [
                'status' 		=> 'success',
                'message' 		=> __('Successfully added', 'wpfnl'),
                'order_bump'	=> true,
                'html'			=> $html,
                'course'		=> $course,
                'user_id'		=> $user_id,
                'step_id'		=> $step_id,
                'paypal_nonce'  => $paypal_nonce,
                'stripe_nonce'  => $stripe_nonce,
            ];
            do_action( 'wpfunnels/order_bump_accepted', $step_id, $product_id );
            return $data;
        }

        return false;
    }


    /**
     * When order bump reject for lms
     * 
     * @param 
     * @return Array
     */
    private function wpfnl_order_bump_reject( $step_id, $product_id, $quantity, $key, $user_id, $funnel_id ){

        if( Wpfnl_lms_learndash_functions::is_learndash_active() ){
            $course = Wpfnl_lms_learndash_functions::get_course_details($step_id);
            $html = $this->render_main_product_design( $course , $step_id, $funnel_id );

            $paypal_nonce = learndash_paypal_init_user_purchase_hash( $user_id, $course['id']);
            $stripe_nonce = wp_create_nonce( 'stripe-nonce-' . $course['id'] . $course['price']*100 . $course['type'] );
            update_user_meta(
                1,
                'ld_purchase_nonce_' . $paypal_nonce,
                array(
                    'user_id'    => $user_id,
                    'product_id' => $course['id'],
                    'time'       => time(),
                    'nonce'      => $paypal_nonce,
                )
            );
            $data = [
                'status' => 'success',
                'message' => __('Successfully removed', 'wpfnl'),
                'html'			=> $html,
                'course'		=> $course,
                'user_id'		=> $user_id,
                'step_id'		=> $step_id,
                'paypal_nonce'  => $paypal_nonce,
                'stripe_nonce'  => $stripe_nonce,
            ];
            do_action( 'wpfunnels/order_bump_rejected', $step_id, $product_id );
            return $data;
        }
        return [];
    }


    /**
	 * render main product design
	 */
	private function render_main_product_design(  $course , $step_id, $funnel_id ){

		$course_title 		= $course['title'];
		$description 		= $course['description'];
		$no_of_lesson 		= $course['no_of_lesson'];
		$course_type 		= $course['type'];
		$billing_cycle 		= $course['billing_cycle'];
		$recurring_time 	= $course['recurring_time'];
		$price 				= $course['price'];
		$trial_price 		= $course['trial_price'];
		$trial_period 		= $course['trial_period'];
		$is_expire 			= $course['is_expire'];
		$expire_days 		= $course['expire_days'];
		$image 				= $course['image'];
		$billing_cycle_unit = $course['billing_cycle_unit'];
		$trial_period_unit  = $course['trial_period_unit'];
		$currency  = $course['currency'];
		$discount_price  	= isset($course['discount_price']) ? $course['discount_price'] : '';

		
		//----course type------
		if( $course_type == 'subscribe' ) {
			$course_type_text = 'Subscription';

		}else if( $course_type == 'open' || $course_type == 'free' ){
			$course_type_text = 'Free';

		}else if( $course_type == 'paynow' ){
			$course_type_text = 'One-time Payment';

		}else {
			$course_type_text = 'Closed';
		}

		//----billing cycle unit------
		if( $course['billing_cycle_unit'] == 'Y' ) {
			$billing_cycle_unit_text = 'year';

		}else if( $course['billing_cycle_unit'] == 'M' ){
			$billing_cycle_unit_text = 'month';

		}else if( $course['billing_cycle_unit'] == 'W' ){
			$billing_cycle_unit_text = 'week';

		}else {
			$billing_cycle_unit_text = 'day';
		}

		//----trial period unit------
		if( $course['trial_period_unit'] == 'Y' ) {
			$trial_period_unit_text = 'year';

		}else if( $course['trial_period_unit'] == 'M' ){
			$trial_period_unit_text = 'month';

		}else if( $course['trial_period_unit'] == 'W' ){
			$trial_period_unit_text = 'week';

		}else {
			$trial_period_unit_text = 'day';
		}
		ob_start();
		do_action( 'wpfunnels/before_checkout_form', $step_id );
		?>
		<div class="lms-checkout-box">
			<div class="lms-checkout-header">
				<h4>Order Details</h4>
			</div>
			<div class="lms-checkout-body">
				<div class="course-image">
					<img src="<?php echo $image; ?>" alt="course image" />
				</div>

				<div class="course-content">
					<h2 class="course-title">
						<?php echo $course_title ?>
						<span class="no-of-lesson">- <?php echo $no_of_lesson ?> Lessons</span>
					</h2>

					<div class="lms-single-block course-details-block">
						<h4 class="lms-block-title">Course Details</h4>
						<div class="course-description">
							<?php echo $description ?>
						</div>
					</div>

					<div class="lms-single-block course-plan-block">
						<h4 class="lms-block-title">Plan Details</h4>
						<div class="course-plan">
							<?php
								if( $course_type == 'subscribe' ){
									echo $course_type_text .' - <span class="primary-color">'.$price.'/'.$billing_cycle_unit_text.'</span>';

									if( !empty( $trial_period ) ){
										echo '<small> ('.$trial_period.' '.$trial_period_unit_text.' trial period)</small>';
									}

								} else if( $course_type == 'paynow' ){
									
									if( $discount_price ){
										echo $course_type_text .' - <span class="primary-color"><del>'.$currency.$price.'</del>  <ins style="text-decoration:none">'.$currency.$discount_price.'</ins></span>';
									}else{
										echo $course_type_text .' - <span class="primary-color">'.$currency.$price.'</span>';
									}
									if( $is_expire == 'on' ){
										echo '<small> (expires in '.$expire_days.' days)</small>';
									}
								}else{
									echo $course_type_text;

									if( $is_expire == 'on' ){
										echo '<small> (Login or register to enroll) (expires in '.$expire_days.' days)</small>';
									}
								}
							?>
						</div>
					</div>
				</div>
			</div>
		</div>
		<div class="lms-checkout-footer">
			<?php
			if( $course_type == 'subscribe' ){
				if( !empty( $trial_period ) ){
					?>

					<p class="footer-title">Course Trial Fees <span class="footer-price"><?php echo $currency.$trial_price; ?></span></p>

				<?php }else{ ?>

					<p class="footer-title">Course Subscription Fees <span class="footer-price"><?php echo $currency.$price; ?></span></p>
					<?php
				}
			}else if( $course_type == 'paynow' ){
				if( $discount_price ){ ?>
					<p class="footer-title">Course Fees <span class="footer-price"><del><?php echo $currency.$price; ?></del> <ins style="text-decoration:none"> <?php echo $currency.$discount_price; ?> </ins></span></p>
				<?php } else{ ?>
					<p class="footer-title">Course Fees <span class="footer-price"><?php echo $currency.$price; ?></span></p>
				<?php }
				?>
			<?php }else{ ?>
				<p class="footer-title">Course Fees <span class="footer-price">Free</span></p>
			<?php } ?>

			<!-- buy now button -->
			<?php
			if (is_user_logged_in() ){
				$course_access = sfwd_lms_has_access( $course['id'], get_current_user_id() );
				$next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url($funnel_id,$step_id).'?wpfnl_ld_payment=free&wpfnl_course_status=enrolled';
				$lms_button_text 		= get_option( 'learndash_settings_custom_labels' );
				$button_text 			= !empty($lms_button_text['button_take_this_course']) ? $lms_button_text['button_take_this_course'] : 'Take This Course';
				if ($course_access ){
					echo '<a class="btn-default" href="'.$next_step_url.'" id="wpfnl-lms-access-course">'.$button_text.'</a>';
					echo '<span class="wpfnl-lms-access-course-message"></span>';
				}else if($course_type == 'free'){
					$next_step_url = Wpfnl_lms_learndash_functions::get_next_step_url($funnel_id,$step_id);
					echo '<a class="btn-default" href="'.$next_step_url.'" user_id="'.get_current_user_id().'" step_id="'.$step_id.'" course_id="'.$course['id'].'" id="wpfnl-lms-free-course">'.$button_text.'</a>';
					echo '<span class="wpfnl-lms-free-course-message"></span>';
				}
				else{
					echo do_shortcode('[learndash_payment_buttons course_id='.$course['id'].']');
				}
			}else{
				echo do_shortcode('[learndash_login]');
			}
			?>
		</div>
		<?php
	
		return ob_get_clean();
	}
}