<?php


namespace WPFunnels\Modules\Frontend\LearnDash;

use WPFunnels\Wpfnl_functions;
use WPFunnels\lms\helper\Wpfnl_lms_learndash_functions;

class Wpfnl_Frontend_LearnDash
{
	public $funnel_id;

	public $step_id;

	// const CHECKOUT = "";

	public function __construct()
	{

		$this->validations = [
			'logged_in' => true,
			'user_can' => 'manage_options',
		];


        add_filter( 'learndash_paypal_purchase_success_url', [$this, 'customize_redirect_url'], 10, 2);
		add_filter( 'learndash_stripe_purchase_success_url', [$this, 'customize_redirect_url'], 10, 2);
		add_filter( 'learndash_payment_button', [$this, 'add_step_id_learndash_payment_button'], 10, 2 );
		add_filter( 'learndash_paypal_payment_button', [$this, 'learndash_paypal_payment_button'], 10, 2 );
		add_filter( 'learndash_login_shortcode_atts', [$this, 'learndash_login_shortcode_atts'], 10, 1 );

		add_action( 'wp_ajax_wpfnl_learndash_already_enroll_course', [$this, 'wpfnl_learndash_already_enroll_course']);
		add_action( 'wp_ajax_nopriv_wpfnl_learndash_already_enroll_course', [$this, 'wpfnl_learndash_already_enroll_course']);

		add_action( 'wp_ajax_wpfnl_learndash_free_course', [$this, 'wpfnl_learndash_free_course']);
		add_action( 'wp_ajax_nopriv_wpfnl_learndash_free_course', [$this, 'wpfnl_learndash_free_course']);
		add_action( 'learndash_login_model_register_url', [$this, 'wpfnl_learndash_login_model_register_url']);

		add_action( 'wp_ajax_wpfnl_get_product_info', [$this, 'wpfnl_get_product_info']);
		add_action( 'wp_ajax_nopriv_wpfnl_get_product_info', [$this, 'wpfnl_get_product_info']);

		add_action( 'learndash_get_price_by_coupon', [$this, 'wpfnl_learndash_get_price_by_coupon']);

	}

    /**
	 * Customize learndash redirect url after payment successful
	 */
	public function customize_redirect_url( $redirect_url, $course_id ){

		$step_id = isset($_GET['step_id']) ? $_GET['step_id'] : '';
		$user_id = isset($_POST['stripe_user_id']) ? $_POST['stripe_user_id'] : '';
		if( !$step_id ){
			if( isset($_POST['custom']) ){
				$pos = strpos($_POST['custom'], '&');
				if( $pos !== false ){
					$custom_values = explode('&',$_POST['custom']);
					$step_id = isset($custom_values[0]) ? $custom_values[0] : '';
				}else{
					$step_id = isset($_POST['custom']) ? $_POST['custom'] : '';
				}
			}else{
                $step_id = get_the_ID();
            }
			
			
		}
        
		if( !$user_id ){
			if( isset($_GET['user_id']) ){
				$user_id = $_GET['user_id'];
			}else{
				if( isset($_POST['custom']) ){
					$pos = strpos($_POST['custom'], '&');
					if( $pos !== false ){
						$custom_values = explode('&',$_POST['custom']);
						$user_id = isset($custom_values[1]) ? $custom_values[1] : '';
					}
				}
			}
		}
	
		if( $step_id ){
          
			$funnel_id = get_post_meta( $step_id, '_funnel_id', true );
			$step_type = get_post_meta( $step_id, '_step_type', true );
			if( $funnel_id ){

				$next_step = Wpfnl_functions::get_next_step( $funnel_id, $step_id );

				if( isset($next_step['step_id']) ){
					if( $next_step['step_type'] == 'conditional' ){
						$next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step['step_id'] );
					}

					$redirect_url = get_permalink( $next_step['step_id']);
					$payment_method = 'Free';
					if( isset( $_POST['stripe_token_id'] ) ){

						if( 'upsell' == $next_step['step_type'] || 'downsell' == $next_step['step_type'] || 'thankyou' == $next_step['step_type'] ){
							if( str_contains($redirect_url, '?') ){
								$redirect_url = $redirect_url . '&wpfnl_ld_payment=stripe';
							}else{
								$redirect_url = $redirect_url . '?wpfnl_ld_payment=stripe';
							}
							$payment_method = 'Stripe';
						}
						
					}
					else{

						if( 'upsell' == $next_step['step_type'] || 'downsell' == $next_step['step_type'] || 'thankyou' == $next_step['step_type'] ){
							if( str_contains($redirect_url, '?') ){
								$redirect_url = $redirect_url . '&wpfnl_ld_payment=paypal';
							}else{
								$redirect_url = $redirect_url . '?wpfnl_ld_payment=paypal';
							}
							$payment_method = 'Paypal';
						}

					}
					$discounted_price = Wpfnl_lms_learndash_functions::get_discounted_course_price($step_id);
					$order_details = [
						'step_id' 			=> $step_id,
						'funnel_id' 		=> $funnel_id,
						'user_id' 			=> $user_id,
						'course_details' 	=> Wpfnl_lms_learndash_functions::get_course_details_by_id($course_id),
						'discounted_price' 	=> $discounted_price,
					];

					$this->create_lms_order( $order_details , $payment_method );
				}
			}
		}
       
		return $redirect_url;
	}


	private function create_lms_order( $order_details = [], $payment_method = '' ){
		global $wpdb;
		$wpfnl_lms_order_table = $wpdb->prefix . 'wpfnl_lms_order';
		$funnel_id             = $order_details[ 'funnel_id' ];
		$step_id               = $order_details[ 'step_id' ];
		$user_id               = $order_details[ 'user_id' ];
		$course_id             = $order_details[ 'course_details' ]['id'];
		$amount = 0;
		if( isset($order_details['course_details']['type']) && 'subscribe' === $order_details['course_details']['type'] ){
			if( isset($order_details['course_details']['trial_price']) && $order_details['course_details']['trial_price'] ){
				$amount = $order_details[ 'course_details' ]['trial_price'];
			}
		}else{
			$amount = isset($order_details[ 'discounted_price' ]) && $order_details[ 'discounted_price' ] ? $order_details[ 'discounted_price' ] : $order_details[ 'course_details' ]['price'];
		}
		
		$query = $wpdb->prepare( "SELECT * FROM {$wpfnl_lms_order_table} WHERE funnel_id = %d AND step_id = %d AND user_id = %d", $funnel_id, $step_id, $user_id );
		$result = $wpdb->get_results( $query );

		if( !empty($result) ){
			$query = $wpdb->prepare("UPDATE {$wpfnl_lms_order_table} SET meta='order' WHERE funnel_id = %d AND step_id = %d AND user_id = %d", $funnel_id, $step_id, $user_id);
			$wpdb->query( $query );
		}

		$step_type= get_post_meta( $step_id, '_step_type', true );

		if( 'checkout' === $step_type ){
			$main_product = get_post_meta($step_id,'_wpfnl_checkout_products',true);
			$key = array_search($course_id, array_column($main_product, 'id'));
			if( false !== $key ){
				$step_type = 'main_product';
			}else{
				$step_type = 'orderbump';
			}
		}

		$order_data = array(
			'funnel_id'    => $order_details[ 'funnel_id' ],
			'step_id'      => $order_details[ 'step_id' ],
			'user_id'      => $order_details[ 'user_id' ],
			'meta'         => 'new_order',
			'payment_method'=> $payment_method,
			'course_id'		=> $course_id,
			'amount'		=> $amount,
			'step_type'		=> $step_type,
			'order_data'   => serialize( $order_details[ 'course_details' ] ),
			'date_created' => date( "Y-m-d h:i:s" )
		);

		$wpdb->insert(
			$wpfnl_lms_order_table,
			$order_data
		);
		$order_id = $wpdb->insert_id;
		
		$this->set_automation_cookie($order_details[ 'funnel_id' ], $order_details[ 'step_id' ], $step_type, $order_id );

	}


	/**
	 * Set automation cookie for lms
	 */
	private function set_automation_cookie( $funnel_id = '', $step_id = '', $step_type = '', $order_id = '' ){
		$cookie_name        = 'wpfunnels_automation_data';
        $cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
		if( $step_type ){

			if( 'main_product' === $step_type ){
				if( $order_id ){
					$cookie['main_order_id']    = $order_id;
            		$cookie['funnel_id']        = $funnel_id;
				}
			}elseif( 'upsell' === $step_type || 'downsell' === $step_type ){
				$offer_settings = \WPFunnels\Wpfnl_functions::get_offer_settings();
				$cookie['child_order']   = $offer_settings['offer_orders'] == 'child-order' ? true : false;
				$cookie['offer'][$step_id]   = array(
					'id'            => $step_id,
					'type'          => $step_type,
					'order_id'      => $order_id,
					'status'        => 'accepted'
				);

			}
			if( isset($cookie['ob_accepetd_products']) && !empty($cookie['ob_accepetd_products']) ){
				$cookie['main_order_id']    = false;
			}
			if( isset($cookie['ob_accepetd_products']) && empty($cookie['ob_accepetd_products']) ){
				$cookie['orderbump_accepted']    = false;
			}
			if( !isset($cookie['already_enrolled']) ){
				$cookie['already_enrolled']   = [];
			}
			if(!isset($cookie['funnel_id'])) {
				$cookie['funnel_id']   = $funnel_id;
			}
			do_action('wpfunnels/lms_course_enroll', $cookie , $step_id);
			setcookie( $cookie_name, wp_json_encode( $cookie ), strtotime( '+1 days' ), '/', COOKIE_DOMAIN );
		}
	}


	/**
	 * change paypal button and pass step id as custom
	 *
	 */
	public function add_step_id_learndash_payment_button( $payment_button, $payment_params )
	{

		$step_id = get_the_ID();

		$course  = Wpfnl_lms_learndash_functions::get_course_details( $step_id );
        
		if( $course && !empty( $course ) ) {
			$funnel_id   = Wpfnl_functions::get_funnel_id_from_step( $step_id );
			$course_type = isset( $course[ 'type' ] ) ? $course[ 'type' ] : '';
			$url 	 = get_home_url();
			$user_id = get_current_user_id();
            $payment_button = str_replace( '</form>', '<input type="hidden" name="custom" value="' . $step_id .'&'. $user_id . '"></form>', $payment_button );

			$search  = '<input type="hidden" name="return" value="'.$url.'/sfwd-lms/paypal?';
			$replace = '<input type="hidden" name="return" value="'.$url.'/sfwd-lms/paypal?step_id='.$step_id.'&user_id='.$user_id.'&';
		
			$payment_button = str_replace($search,$replace,$payment_button);
           
			if( (isset($_GET['wpfnl_ld_payment']) && $_GET['wpfnl_ld_payment'] === 'free') ||  $course_type === 'free' ) {

				$next_step = Wpfnl_functions::get_next_step( $funnel_id, $step_id );
				if( isset( $next_step[ 'step_id' ] ) ) {
					if( $next_step[ 'step_type' ] == 'conditional' ) {
						$next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step[ 'step_id' ] );
					}
					$next_step_url = get_permalink( $next_step[ 'step_id' ] ) . '?wpfnl_ld_payment=free';
					$post          = isset( $payment_params[ 'post' ] ) ? $payment_params[ 'post' ] : '';
					$post_id       = isset( $post->ID ) ? $post->ID : '';
					$permalink     = $post_id !== '' ? get_permalink( $post_id ) : '';
					$search        = '<form action="' . $permalink . '" method="post">';
					$replace       = '<form action="' . $next_step_url . '" method="post">';
					$payment_button= str_replace( $search, $replace, $payment_button );
				}
			}else {
				if( isset( $_GET[ 'wpfnl_ld_payment' ] ) ) {
					$remove_gateway = $_GET[ 'wpfnl_ld_payment' ] == 'paypal' ? 'stripe' : 'paypal';
					?>
					<script>
						jQuery(document).ready(function ($) {
							let remove_gateway = '<?php echo $remove_gateway ?>';
							$(".learndash_" + remove_gateway + "_button").remove();
						});
					</script>
					<?php
				}
			}

		}
		return $payment_button;
	}

	/**
	 * Change login Button name for learndash
	 * @param $attributes
	 * @return mixed
	 */

	public function learndash_login_shortcode_atts($attributes){
		$lms_button_text 		= get_option( 'learndash_settings_custom_labels' );
		if( isset( $attributes[ 'login_label' ] ) ) {
			$attributes[ 'login_label' ] = isset($lms_button_text['button_take_this_course']) ? $lms_button_text['button_take_this_course'] : 'Take This Course' ;
		}
		return $attributes;
	}


	/**
	 * Update lms order meta when user is already enrolled
	 *
	 *
	 */
	public function wpfnl_learndash_already_enroll_course(){
		$response = [
			'success' => false,
		];
		
		if( isset($_POST['step_id']) && isset($_POST['user_id']) ){
			$funnel_id = get_post_meta($_POST['step_id'],'_funnel_id',true);
			$user_id   = $_POST['user_id'];
			$step_id   = $_POST['step_id'];

			$step_type = '';
			if( $funnel_id && $user_id && $step_id ){
				Wpfnl_lms_learndash_functions::update_lms_order_status( $funnel_id, $step_id, $user_id );
				$step_type = get_post_meta($step_id,'_step_type',true);
			}
			$response = [
				'success' => true,
			];

			if( $step_type ){
				/** Set Cookie Data */
				$cookie_name        = 'wpfunnels_automation_data';
				$cookie             = isset( $_COOKIE[$cookie_name] ) ? json_decode( wp_unslash( $_COOKIE[$cookie_name] ), true ) : array();
				
				if( isset($cookie['already_enrolled']) ){
					array_push($cookie['already_enrolled'],$step_type);
				}else{
					$cookie['already_enrolled']   = [];
					array_push($cookie['already_enrolled'],$step_type);
				}

				if( 'upsell' === $step_type || 'downsell' === $step_type ){
					$cookie['offer'][$step_id]   = array(
						'id'            => $step_id,
						'type'          => $step_type,
						'status'        => 'accepted'
					);
				}

				if( !isset($cookie['user_info']) ){
					$current_user = wp_get_current_user();
					$cookie['user_info']   = [
						'id'            => get_current_user_id(),
						'first_name'    => $current_user->user_firstname,
						'last_name'     => $current_user->user_lastname,
						'email'         => $current_user->user_email
					];
				}
				if( !isset($cookie['ob_accepetd_products']) ){
					$cookie['ob_accepetd_products'] = [];
				}

				if(!isset($cookie['orderbump_accepted'])){

					if( !empty($cookie['ob_accepetd_products']) ){
						$cookie['orderbump_accepted']   = true;
					}else{
						$cookie['orderbump_accepted']   = false;
					}
				}
				if(!isset($cookie['funnel_id'])) {
					$cookie['funnel_id']   = $funnel_id;
				}
				setcookie( $cookie_name, wp_json_encode( $cookie ), strtotime( '+1 days' ), '/', COOKIE_DOMAIN );
			}

		}
		do_action('wpfunnels/lms_course_enroll', $cookie , $step_id);
		wp_send_json( $response );
	}


	/**
	 * Update lms order meta when user is already enrolled
	 *
	 *
	 */
	public function wpfnl_learndash_free_course(){
		$response = [
			'success' => false,
		];

		if( isset($_POST['step_id']) && isset($_POST['user_id']) ){
			$postData 	= isset($_POST['data']) ? $_POST['data'] : '';
			$post_data  = array();
			parse_str($postData, $post_data);
			$funnel_id = get_post_meta($_POST['step_id'],'_funnel_id',true);
			$user_id   = $_POST['user_id'];
			$step_id   = $_POST['step_id'];
			$payment_method = 'free';
			if( isset($_POST['course_id']) ){
				$courses = [$_POST['course_id']];
				learndash_user_set_enrolled_courses($user_id, $courses);
				$order_details = [
					'step_id' => $step_id,
					'funnel_id' => $funnel_id,
					'user_id' => $user_id,
					'course_details' => Wpfnl_lms_learndash_functions::get_course_details_by_id( $_POST['course_id'] ),
				];
				$this->create_lms_order( $order_details , $payment_method );
				$next_step = Wpfnl_lms_learndash_functions::get_next_step_url( $funnel_id, $step_id );
			}

			$response = [
				'success' => true,
				'next_step' => $next_step
			];
		}

		wp_send_json( $response );
	}


	public function learndash_paypal_payment_button($paypal_button,$button_data){
		$user_id = get_current_user_id();
		$step_id = get_the_ID();

		if( $user_id && $step_id ){
			$url = $button_data['atts']['returnurl'];
			$parse = parse_url($url);
			$url = $parse['scheme']."://".$parse['host'];
			$search  = '<input type="hidden" name="return" value="'.$url.'/sfwd-lms/paypal?';
			$replace = '<input type="hidden" name="return" value="'.$url.'/sfwd-lms/paypal?step_id='.$step_id.'&user_id='.$user_id.'&';
			$paypal_button = str_replace($search,$replace,$paypal_button);
		}
		return $paypal_button;
		
	}


	/**
	 * Modify registration url
	 * 
	 * @param String
	 * @return String
	 * @since 1.0.3
	 * 
	 */
	public function wpfnl_learndash_login_model_register_url( $url ){

		// $funnel_id = get_post_meta(get_the_ID(), '_funnel_id', true );
		// if( $funnel_id ){
		// 	$url .= '?is_wpfunnel=yes&funnel_id='.$funnel_id.'&step_id='.get_the_ID();
		// }
		return $url;
	}


	/**
	 * Get product info with discounted price
	 * 
	 * @return Array
	 * @since 1.0.3
	 */
	public function wpfnl_get_product_info( $payload ){
		
		$step_id = isset($payload['step_id']) ? $payload['step_id'] : '';
		if( $step_id ){
			$step_type = get_post_meta( $step_id, '_step_type', true );
			if( 'checkout' == $step_type ){
				$discount = get_post_meta( $step_id, '_wpfnl_checkout_discount_main_product', true );
				if( is_array($discount) ){
					if( isset($discount['discountOptions']) && isset($discount['mutedDiscountValue']) && 'original' != $discount['discountOptions'] && $discount['mutedDiscountValue'] ){
						if( isset($discount['discountedPrice']) ){
							$discounted_price = $discount['discountedPrice'];
						}
					}
					
				}
			}
		}
		return [
			'success' => true,
		];
	}


	/**
	 * 
	 * Add discount price to course
	 * 
	 * @param String 
	 * @return String
	 * @since 1.0.4
	 */
	public function wpfnl_learndash_get_price_by_coupon( $price ){
		$step_type = get_post_meta( get_the_ID(), '_step_type', true );
		if( $step_type ){
			
			if( 'checkout' == $step_type ){
				$discount = get_post_meta( get_the_ID(), '_wpfnl_checkout_discount_main_product', true );
				if( is_array($discount) ){
					if( isset($discount['discountOptions']) && isset($discount['mutedDiscountValue']) && 'original' != $discount['discountOptions'] && $discount['mutedDiscountValue'] ){
						if( isset($discount['discountedPrice']) ){
							$price = $discount['discountedPrice'];
						}
					}
				}
			}

			if( 'upsell' == $step_type || 'downsell' == $step_type ){
				$discount = get_post_meta( get_the_ID(), '_wpfnl_'.$step_type.'_discount', true );
				$course   = Wpfnl_lms_learndash_functions::get_course_details( get_the_ID() );
				if( is_array( $discount ) ){
					if( isset($discount['discountType']) && isset($discount['discountValue']) && 'original' != $discount['discountType'] && $discount['discountValue'] ){
						$price = Wpfnl_functions::calculate_discount_price( $discount['discountType'], $discount['discountValue'], $course['price'] );
					}
				}
			}
		}

		return $price;
	}

}
