<?php

namespace WPFunnelsPro\Import;
use WPFunnels\Wpfnl_functions;
use WPFunnels\Traits\SingletonTrait;
use WPFunnels\Wpfnl;
use WPFunnels\Data_Store\Wpfnl_Funnel_Store_Data;
use WPFunnels\Data_Store\Wpfnl_Steps_Store_Data;
use lementor\TemplateLibrary\Source_Base;
use WPFunnelsPro\AbTesting\Wpfnl_Ab_Testing;
use Elementor\Controls_Stack;
use Elementor\Plugin;
use Elementor\Utils;

/**
 * This class is responsible import funnel 
 * 
 * Import json file 
 * @since 1.7.5
 */
class Wpfnl_Import {

    
    // use SingletonTrait;


    /**
     * Initialize ajax callback
     * 
     * @since 1.7.5
     */
    public function __construct(){

		add_action('wp_ajax_wpfnl_import_funnels', [$this, 'import_funnels']);
		add_action('wp_ajax_nopriv_wpfnl_import_funnels', [$this, 'import_funnels']);

        wp_ajax_helper()->handle('wpfnl-import-funnels')
            ->with_callback([ $this, 'import_funnels' ])
            ->with_validation($this->get_validation_data());

    }


    /**
     * Import funnel by uploading json file
     * 
     * @param Void
     * 
     * @return Array
     * @since  1.7.5
     */
    public function import_funnels(){
        $file_data = json_decode(file_get_contents($_FILES['uploaded_file']['tmp_name']), true);
        $invalid_data = false;
        if( is_array($file_data) ){
            $funnel 	= Wpfnl::$instance->funnel_store;
            $exclude_meta = array( '_is_imported', '_steps_order', '_steps', 'funnel_data' );
            foreach( $file_data as $key=>$data ){
                if( !isset($data['funnel_id']) ){
                    $invalid_data = true;
                    break;
                }
                $funnel_data = [];
                $funnel_name = isset( $data['funnel_name'] ) ? $data['funnel_name'] : 'New Funnel';
                $funnel_id 	= $funnel->create( $funnel_name );
                $funnel->update_meta($funnel_id, '_is_imported', 'yes');
                foreach( $data['funnel_meta'] as $meta_key=>$meta){
                    if( !in_array( $meta_key, $exclude_meta ) ){
                        $funnel->update_meta($funnel_id, $meta_key,maybe_unserialize($data['funnel_meta'][$meta_key][0]));
                    }else{
                        if( 'funnel_data' === $meta_key ){
                            $funnel_data = maybe_unserialize($data['funnel_meta'][$meta_key][0]);
                        }
                    }
                }
                $_new_step_ids      = [];
                $_new_step_order    = [];
               
                foreach( $data['steps_data'] as $step_meta_key=>$step_data ){
                    $response = $this->import_step( $funnel_id , $step_data );
                    
                    if( isset($response['id'], $response['title'], $response['type']) ){
                        $_new_step_ids[$step_data['id']] = $response['id'];
                        $_new_step_order[] = [
							'id'        => $response['id'],
							'step_type' => $response['type'],
							'name'      => $response['title'],
						];
                        $attachments = get_posts( array(
                            'post_type' => 'attachment',
                            'posts_per_page' => -1,
                            'post_parent' => $response['id'],
                            'exclude'     => get_post_thumbnail_id()
                        ) );

                        do_action('wpfunnels_after_step_import', $response['id'], Wpfnl_functions::get_builder_type() );
                    }else{
                        $invalid_data = true;
                        
                        break;
                    }
                }
                $this->update_funnel_data( $_new_step_ids, $funnel_id, $funnel_data );
                update_post_meta($funnel_id, '_steps_order', $_new_step_order);
                update_post_meta($funnel_id, '_steps', $_new_step_order);
            }
        }

        if( $invalid_data ){
            $response = [
                'success' => false,
            ];
        }else{
            
            $response = [
                'success' => true,
            ];
        }
        
        echo json_encode($response, true);
		die();
    }
    

    /**
     * Import step from json
     * @param Int $funnel_id
     * @param Array $step_data
     * @return
     * @since 1.7.5
     * 
     */
    public function import_step( $funnel_id, $step_data ){
        if( $step_data ){
            $builder_type = Wpfnl_functions::get_builder_type();
            $previous_step_id        = isset( $step_data['id'] ) ? $step_data['id'] : '';
            $title          = isset( $step_data['title'] ) ? $step_data['title'] : '';
            $page_template  = isset( $step_data['page_template'] ) ? $step_data['page_template'] : '';
            $post_content   = isset( $step_data['post_content'] ) ? $step_data['post_content'] : '';
            $step_type      = isset( $step_data['type'] ) ? $step_data['type'] : '';
            $step = new Wpfnl_Steps_Store_Data();
            $step_id = $step->create_step($funnel_id, $title, $step_type, $post_content, true);
            
            if( $step_id ){
                $step->update_meta($step_id, '_funnel_id', $funnel_id);
                $step->update_meta($step_id, '_wp_page_template', $page_template);
                $exclude_meta = array( '_wpfnl_ab_testing_start_settings', '_funnel_id' );
                foreach( $step_data['meta'] as $meta_key=>$meta){
                    if( !in_array( $meta_key, $exclude_meta ) ){
                        $step->update_meta($step_id, $meta_key,maybe_unserialize($step_data['meta'][$meta_key][0]));
                    }
                }
                update_post_meta( $step_id, '_wpfnl_ab_testing_start_settings' , Wpfnl_Ab_Testing::get_default_data( $step_id ));
                // if ( $post_content ) {
                //     // Download and replace images.
                //     $content = $this->get_content( $step_id );
                //     // Update post content.
                //     wp_update_post(
                //         array(
                //             'ID'           => $step_id,
                //             'post_content' => $content,
                //         )
                //     );
                // }
                if ( ( 'gutenberg' === $builder_type )  && class_exists( '\WPFunnels\Batch\Gutenberg\Wpfnl_Gutenberg_Source' ) ) {
                    if ( ini_get( 'allow_url_fopen' ) ) {
                        $gutenberg_source  = new \WPFunnels\Batch\Gutenberg\Wpfnl_Gutenberg_Source;
                        $gutenberg_source->import_single_template( $step_id  );
                    }
                }

                // Elementor Data.
                if ( ( 'elementor' === $builder_type ) && class_exists( '\Elementor\Plugin' ) && class_exists( '\WPFunnels\Batch\Elementor\Wpfnl_Elementor_Source' ) ) {
                    if ( ini_get( 'allow_url_fopen' ) ) {
                        $elementor_source  = new \WPFunnels\Batch\Elementor\Wpfnl_Elementor_Source;
                        $elementor_source->import_single_template( $step_id  );
                    }
                }

                if ( ( 'divi-builder' === $builder_type ) && class_exists( '\WPFunnels\Batch\Divi\Wpfnl_Divi_Source' ) ) {
                    if ( ini_get( 'allow_url_fopen' ) ) {
                        $divi_source  = new \WPFunnels\Batch\Divi\Wpfnl_Divi_Source;
                        $divi_source->import_single_template( $step_id  );
                    }
                }

                if ( ( 'oxygen' === $builder_type ) && class_exists( '\WPFunnels\Batch\Oxygen\Wpfnl_Oxygen_Source' ) ) {
                    if ( ini_get( 'allow_url_fopen' ) ) {
                        $oxygen_source  = new \WPFunnels\Batch\Oxygen\Wpfnl_Oxygen_Source;
                        $oxygen_source->import_single_template( $step_id  );
                    }
                }
                delete_post_meta( $step_id, '_funnel_id' ); 
                update_post_meta( $step_id, '_funnel_id', $funnel_id );
                
                return [
                    'id'    => $step_id,
                    'type'  => $step_type,
                    'title' => $title,
                ];
            }
        }
        return false;   
    }


    


    /**
     * Update funnel data and steps
     * 
     * @param Int $funnel_id
     * @param Array $step_data
     * 
     * @return void 
     * @since  1.7.5
     */ 
    public function update_funnel_data( $new_step_ids, $funnel_id, $funnel_data ){
        $funnel_identifier = array();
		if ($funnel_data) {
			$node_data = isset($funnel_data['drawflow']['Home']['data']) ? $funnel_data['drawflow']['Home']['data'] : [];
			foreach ($node_data as $node_key => $node_value) {
				if(isset($node_value['data']['step_id'])) {
					if ( isset($new_step_ids[$node_value['data']['step_id']]) ) {
						$post_edit_link = base64_encode(get_edit_post_link($new_step_ids[$node_value['data']['step_id']]));
						$post_view_link = base64_encode(get_post_permalink($new_step_ids[$node_value['data']['step_id']]));
						$funnel_data['drawflow']['Home']['data'][$node_key]['data']['step_id'] = $new_step_ids[$node_value['data']['step_id']];
						$funnel_data['drawflow']['Home']['data'][$node_key]['data']['step_edit_link'] = $post_edit_link;
						$funnel_data['drawflow']['Home']['data'][$node_key]['data']['step_view_link'] = $post_view_link;
						$funnel_data['drawflow']['Home']['data'][$node_key]['html'] = $node_value['data']['step_type'] . $new_step_ids[$node_value['data']['step_id']];
						$funnel_identifier[$node_value['id']] = $new_step_ids[$node_value['data']['step_id']];
					} else {
						if ($node_value['data']['step_type'] != 'conditional') {
							$funnel_identifier[$node_value['id']] = $node_value['data']['step_id'];
						} else {
							$funnel_identifier[$node_value['id']] = $node_value['data']['node_identifier'];
						}
					}
				}
                if ($node_value['data']['step_type'] == 'conditional') {
                    $funnel_identifier[$node_value['id']] = $node_value['data']['node_identifier'];
                }
			}
		}
		update_post_meta($funnel_id, 'funnel_data', $funnel_data);
		if ($funnel_identifier) {
			$funnel_identifier_json = json_encode($funnel_identifier, JSON_UNESCAPED_SLASHES);
			update_post_meta($funnel_id, 'funnel_identifier', $funnel_identifier_json);
		}
    }



    public function get_validation_data()
    {
        return [
            'logged_in' => true,
            'user_can' => 'manage_options',
        ];
    }
}