<?php
namespace WPFunnelsPro\Frontend;

use WPFunnels\Traits\SingletonTrait;
use WPFunnels\Wpfnl_functions;
use WPFunnelsPro\Wpfnl_Pro;
use WPFunnelsPro\Wpfnl_Pro_functions;
use WPFunnelsPro\Frontend\Modules\Gateways\Payment_Gateways_Factory;
use Wpfnl_Pro_GBF_Offer_Conditions_Factory;

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://rextheme.com
 * @since      1.0.0
 *
 * @package    Wpfnl_Pro
 * @subpackage Wpfnl_Pro/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Wpfnl_Pro
 * @subpackage Wpfnl_Pro/public
 * @author     RexTheme <support@rextheme.com>
 */
class Wpfnl_Pro_Public {

    private static $instance;

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;


    /**
     * Get class instance.
     *
     * @return object Instance.
     */
    public static function getInstance() {
        if (!self::$instance) {
            // new self() will refer to the class that uses the trait
            self::$instance = new self();
        }

        return self::$instance;
    }

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct( ) {
        add_action( 'wp_enqueue_scripts', array($this, 'enqueue_styles'),9999);
        add_action( 'wp_enqueue_scripts', array($this, 'enqueue_scripts'),9999 );
        add_action( 'wp_footer', array($this, 'load_offer_loader') );
        add_action( 'init', array( $this, 'init_funnel' ), 9999 );

        /** this is the hook where we checked if the funnel has any offer step for payments. */
        add_action( 'woocommerce_pre_payment_complete', array( $this, 'maybe_setup_offer_funnel' ), 9999, 1 );

        add_filter( 'wpfunnels/next_step_url', array( $this, 'get_next_step_url' ), 10, 3 );
        add_action( 'wp', [$this, 'wpfnl_get_ab_testing_variation'], 10 );
        add_action( 'wpfunnels/next_step_data', [$this, 'wpfnl_next_step_data_modification'], 10 );
        // add_filter( 'wpfunnels/next_step_data', array( $this, 'modify_next_step_data' ), 10 );

        add_filter( 'wpfunnels/modify_next_step_based_on_order', array( $this, 'modify_next_step_based_on_order' ), 10, 2 );

        add_action( 'wpfunnels/offer_accepted', array( $this, 'add_accepted_offer_details_to_logger' ), 10, 2 );
        add_action( 'wpfunnels/offer_rejected', array( $this, 'add_rejected_offer_details_to_logger' ), 10, 2 );

        wp_ajax_helper()->handle('wpfnl-get-variation-price')
            ->with_callback([ $this, 'wpfnl_get_variation_price' ]);


	}


    /**
     * get next step url
     *
     * @param $thankyou_step_url
     * @param $next_step_url
     * @param $order
     * @return mixed
     *
     * @since 2.2.6
     */
	public function get_next_step_url( $thankyou_step_url, $next_step_url, $order ) {
	    // check if payment gateway is supported or not
        $offer_settings = Wpfnl_functions::get_offer_settings();

        if( $offer_settings['skip_offer_step_for_free'] === 'on' ){
            if( !($order->get_total() > 0) ){
                return $thankyou_step_url;
            }   
            
        }
        if( $offer_settings['skip_offer_step'] === 'on' ){
            $supported_gateways = Payment_Gateways_Factory::getInstance()->get_supported_payment_gateways();
            if( isset($supported_gateways[$order->get_payment_method()]) ){
                return $next_step_url;
            }else{
                return $thankyou_step_url;
            }
        }
	    return $next_step_url;
    }


    /**
     * set plugin name
     *
     * @param $plugin_name
     */
	public function set_name( $plugin_name ) {
        $this->plugin_name = $plugin_name;
    }

    /**
     * set plugin version
     * @param $version
     */
    public function set_version( $version ) {
        $this->version = $version;
    }

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wpfnl_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wpfnl_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'assets/css/wpfnl-pro-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wpfnl_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wpfnl_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
        if( is_single() && get_post_type() === WPFNL_STEPS_POST_TYPE ) {
            wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'assets/js/wpfnl-pro-public.js', array('jquery'), $this->version, true);
        }
	}


    /**
     * load offer loader screen
     */
	public function load_offer_loader() {
        if( Wpfnl_Pro_functions::is_upsell_downsell_step() ) {
            ?>
            <div class="wpfunnels-offer-loader">
                <div class="wpfunnels-offer-loader-wrapper">
                    <div class="wpfnl-loader"></div>
                    <h2 class="loader-ittle">
                        Loading
                        <span class="dot-wrapper">
                            <span class="dot-one">.</span> 
                            <span class="dot-two">.</span> 
                            <span class="dot-three">.</span>
                        </span>
                    </h2>
                    <p class="description"><?php echo __('Please wait while processing your order', 'wpfnl-pro'); ?></p>
                </div>
            </div>
            <?php
        }
    }


	public function init_funnel() {
	    $this->initialize_funnel_session();
    }



    /**
     * start session for a funnel
     *
     * @since 1.0.0
     */
    private function initialize_funnel_session() {
	    if(Wpfnl_functions::is_funnel_step_page()) {
	        $funnel_id  =  Wpfnl_functions::get_funnel_id();
            if ( ! $funnel_id ) {
                return;
            }
            if ( Wpfnl_functions::check_if_this_is_step_type('landing') || Wpfnl_functions::check_if_this_is_step_type('checkout') ) {
                $data = array(
                    'funnel_id'     => $funnel_id,
                    'steps'         => get_post_meta( $funnel_id, '_steps_order', true ),
                );
                Wpfnl_Pro::instance()->session->set_session( $funnel_id, $data );
            }
            elseif ( Wpfnl_functions::check_if_this_is_step_type('thankyou') ) {
                Wpfnl_Pro::instance()->session->destroy_session( $funnel_id );
            }
            elseif ( Wpfnl_functions::check_if_this_is_step_type('upsell') || Wpfnl_functions::check_if_this_is_step_type('downsell') ) {
                if ( ! ( is_user_logged_in() && current_user_can( 'manage_options' ) ) ) {
                    if ( ! Wpfnl_Pro::instance()->session->is_active_session( $funnel_id ) ) {
                        wp_die( esc_html__( 'Your session is expired', 'wpfnl-pro' ) );
                    }
                }
            }

        }
    }


    /**
     * may be setup offer funnel
     *
     * @param string $order_id
     *
     * @since 2.2.6
     */
    public function maybe_setup_offer_funnel( $order_id = '' ) {
        if ( empty( $order_id ) ) {
            return;
        }
        $order = wc_get_order( $order_id );
        $this->start_offer_funnel($order);
    }


    /**
     * setup offer funnels
     *
     * @param string $order
     */
    public function start_offer_funnel( $order ) {
        $is_offer_funnel 	= Wpfnl_Pro_functions::is_offer_exists( $order );
        if( $is_offer_funnel ) {
            $order_gateway          = $order->get_payment_method();
            $payment_gateway_obj    = Wpfnl_Pro::instance()->payment_gateways->build_gateway($order_gateway);
            if ( $payment_gateway_obj ) {
                do_action( 'wpfunnels/offer_funnel_started', $order );
            }
        }
    }


    /**
     * 
     */
    public function wpfnl_get_variation_price($payload){
       
        $response = [
            'success' => false,
            'data'    => 'data not found'
        ];
        if( isset($payload['product_id']) && $payload['product_id'] ){
            if( isset($payload['attr']) && $payload['attr'] ){

                $variation_id = (new \WC_Product_Data_Store_CPT())->find_matching_product_variation(
                    new \WC_Product($payload['product_id']),
                    $payload['attr']
                );

                if( $variation_id ){
                   $product = wc_get_product($variation_id);
                }else{
                    $product = wc_get_product($payload['product_id']);
                }
                $response = [
                    'success' => true,
                    'data'    => $product ? $product->get_price_html() : '',
                ];
            }
        }
        return $response;
    }


    /**
     * Get AB testing variation for view in frontend
     * 
     * @since 1.7.3
     * @return @void
     */
    public function wpfnl_get_ab_testing_variation(){
        $get = Wpfnl_Pro_functions::get_sanitized_get_post();
        if( isset($get['get']['wpfnl-step-id']) ){
            if (class_exists('\WPFunnelsPro\AbTesting\Wpfnl_Ab_Testing')) {
                $instance = new \WPFunnelsPro\AbTesting\Wpfnl_Ab_Testing;
                $function_exist = is_callable(array($instance, 'get_ab_testing_variation_id'));
                if( $function_exist ){
                    $step_id = $get['get']['wpfnl-step-id'];
                    $funnel_id = Wpfnl_functions::get_funnel_id_from_step( $step_id );
                    $displayable_variation_id = \WPFunnelsPro\AbTesting\Wpfnl_Ab_Testing::get_ab_testing_variation_id( $funnel_id, $step_id );
                    if( $displayable_variation_id ){
                        $url = get_the_permalink($displayable_variation_id);
                        $utm_settings 		= Wpfnl_functions::get_funnel_utm_settings( $funnel_id );
                        if($utm_settings['utm_enable'] == 'on') {
                            unset($utm_settings['utm_enable']);
                            $url = add_query_arg($utm_settings,$url);
                            $url   = strtolower($url);
                        }
                        wp_safe_redirect( $url );
                        exit;
                    }
                }
            }
        }
    }

    /**
     * Modify next step id for AB testings
     * 
     * @param Array
     * @return Array
     * @since 2.4.21
     */
    public function wpfnl_next_step_data_modification( $data ){
        if (class_exists('\WPFunnelsPro\AbTesting\Wpfnl_Ab_Testing')) {
            $instance = new \WPFunnelsPro\AbTesting\Wpfnl_Ab_Testing;
            $function_exist = is_callable(array($instance, 'get_ab_testing_variation_id'));
            if( $function_exist ){
                if( isset($data['step_id']) ){
                    
                    $funnel_id = get_post_meta( $data['step_id'], '_funnel_id', true );
                    if( $funnel_id ){
                        $displayable_variation_id = \WPFunnelsPro\AbTesting\Wpfnl_Ab_Testing::get_ab_testing_variation_id( $funnel_id, $data['step_id'] );
                        
                        if( $displayable_variation_id ){
                            $data = [
                                'step_id'   => $displayable_variation_id,
                                'step_type' => get_post_meta( $displayable_variation_id, '_step_type', true )
                            ];
                        }
                    }
                }
            }
        }
        return $data;
    }

    /**
     * @desc Skip the next offer step(s)
     * if the assigned product is out of stock.
     * @since 1.6.21
     * @param $next_step
     * @return mixed
     */
    public function modify_next_step_data( $next_step ) {
       
        if( isset( $next_step['step_type'] ) && ( 'upsell' === $next_step['step_type'] || 'downsell' === $next_step['step_type'] ) ) {
            $next_step_type = $next_step[ 'step_type' ];
            $next_step_id = $next_step[ 'step_id' ] ?? false;
            if( $next_step_id ) {
                $funnel_id = get_post_meta( $next_step_id, '_funnel_id', true );
                $is_gbf = get_post_meta( $funnel_id, 'is_global_funnel', true );
                if( 'yes' !== $is_gbf ){
                    $offer_products = Wpfnl_Pro_functions::get_offer_product( $next_step_id, $next_step_type );
                    $product_ids = array_column( $offer_products, 'id' );
                    
                    if( !empty( $product_ids ) ) {
                        foreach( $product_ids as $id ) {
                            $product = '';
                            if( is_plugin_active('wpfunnels-pro-gbf/wpfnl-pro-gb.php' )){
                                $product = wc_get_product( $id );
                            }
                            
                            if( $product && ( $product->is_on_backorder() || $product->is_in_stock() || $product->get_stock_quantity() ) ) {
                                continue;
                            }

                            $funnel_id = Wpfnl_functions::get_funnel_id_from_step( $next_step_id );
                            $next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step_id );
                            return $this->modify_next_step_data( $next_step );
                        }
                    }else{ 
                        $funnel_id = Wpfnl_functions::get_funnel_id_from_step( $next_step_id );
                        $next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step_id );
                        return $this->modify_next_step_data( $next_step );
                    }
                }else{
                    if( is_plugin_active('wpfunnels-pro-gbf/wpfnl-pro-gb.php' )){
                        $offer_product = $this->skip_next_step_for_gbf( $funnel_id, $next_step_id );
                        if( isset($offer_product['id']) && $offer_product['id'] ){
                            $product = wc_get_product( $offer_product['id'] );
                            if( $product && ( $product->is_on_backorder() || $product->is_in_stock() || $product->get_stock_quantity() ) ) {
                                return $next_step;
                            }
                            $funnel_id = Wpfnl_functions::get_funnel_id_from_step( $next_step_id );
                            $next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step_id );
                            return $this->modify_next_step_data( $next_step );
                            
                        }else{
                            $funnel_id = Wpfnl_functions::get_funnel_id_from_step( $next_step_id );
                            $next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step_id );
                            return $this->modify_next_step_data( $next_step );
                        }
                    }
                    
                }
                
            }
        }
        
        return $next_step;
    }


    /**
     * @desc Skip the next offer step(s) for gbf
     * if the assigned product is out of stock.
     * @since 1.6.21
     * @param $next_step
     * @return mixed
     */
    private function skip_next_step_for_gbf( $funnel_id, $step_id ){
        
        $step_type          = get_post_meta($step_id, '_step_type', true);
        $offer_product      = [];
        $order_id           = ( isset( $_POST['order_id'] ) ) ? intval( $_POST['order_id'] ) : (( isset( $_GET['wpfnl-order'] ) ) ? intval( $_GET['wpfnl-order'] ) : 0);
        $gbf_product        = $this->get_gbf_product_from_cookie();
        $gb_funnel_settings = wpfnl()->meta->get_funnel_meta( $funnel_id, 'global_funnel_start_condition' );
        if( $gb_funnel_settings && is_array( $gb_funnel_settings ) ) {
            $offer_mappings  = wpfnl()->meta->get_funnel_meta( $step_id, "global_funnel_{$step_type}_rules" );
            if( isset($offer_mappings['type']) ){
                if( 'specificProduct' == $offer_mappings['type'] ){
                    $product = isset( $offer_mappings['show'] ) && $offer_mappings['show'] ? wc_get_product( $offer_mappings['show'] ) : '';
                    if( $product ){
                        if( $product->get_type() == 'variable' ){
                            return $offer_product;
                        }
                    }
                }
                $param_type = Wpfnl_Pro_GBF_Offer_Conditions_Factory::build($offer_mappings['type']);
                $offer_product   = $param_type->get_offer_product( $offer_mappings, $order_id, $step_id, $gbf_product );
            }
        }
        return $offer_product;
    }

    /**
     * Get GBF product from cookie data
     * 
     * @return Array
     */
    private function get_gbf_product_from_cookie() {
        if( Wpfnl_functions::is_wc_active() ){
            return WC()->session->get('wpfunnels_global_funnel_specific_product') ? WC()->session->get('wpfunnels_global_funnel_specific_product') : [];
        }
        return [];
    }


    /**
     * Add offer details to logger after accepted
     * @param Obj $order
     * @param Array $offer_product
     * 
     * @return void
     * @since 1.6.28
     */
    public function add_accepted_offer_details_to_logger( $order, $offer_product ){
        $this->prepare_offer_data_for_logger( $offer_product, 'Accepted' );
    }
    
    
    /**
     * Add offer details to logger after rejected
     * @param Obj $order
     * @param Array $offer_product
     * 
     * @return void
     * @since 1.6.28
     */
    public function add_rejected_offer_details_to_logger( $order, $offer_product ){
        $this->prepare_offer_data_for_logger( $offer_product, 'Rejected' );
    }


    /**
     * Prepare offer product data for logger
     * 
     */
    private function prepare_offer_data_for_logger( $offer_product, $status ){
        
        $wp_function =  new \WPFunnels\Wpfnl_functions();
        if( is_callable(array($wp_function, 'maybe_logger_enabled')) ){
            if (class_exists("Wpfnl_Logger")) {
                if( Wpfnl_functions::maybe_logger_enabled() && Wpfnl_functions::is_wc_active() ) {
                    $product_array[] = [
                        'name'  => isset($offer_product['name'])    ? $offer_product['name']    : '',
                        'price' => isset($offer_product['price'])   ? $offer_product['price']   : '',
                    ];

                    ob_start();
                    print_r($product_array);
                    $textual_representation = ob_get_contents();
                    ob_end_clean();

                    \Wpfnl_Logger::modify_log_file( 'event',$textual_representation,'Offer Product '.$status );
                }
            }
        } 
    }

    /**
     * Modify next step based on order
     * 
     * @param array  $next_step next step data.
     * @param object $order order details.
     * 
     * @return array
     * @since  1.7.5
     */
    public function modify_next_step_based_on_order( $next_step, $order ){
        if( $order && $next_step ){
            $offer_steps = [ 'upsell', 'downsell', 'conditional' ];
            if( isset($next_step['step_type']) && in_array( $next_step['step_type'], $offer_steps ) ){
                $funnel_id = Wpfnl_functions::get_funnel_id_from_step( $next_step['step_id'] );
                $offer_settings = Wpfnl_functions::get_offer_settings();
                $maybe_skip = get_post_meta($funnel_id,'_wpfunnels_skip_recurring_offer',true);
                $maybe_skip_within_days = get_post_meta($funnel_id,'_wpfunnels_skip_recurring_offer_within_days',true);
                if( 'yes' == $maybe_skip ){
                    $email = $order->get_billing_email();
                    if( $email ) {
                        $order_arg = array(
                            'customer' => $email,
                            'limit'    => -1
                        );
                        $is_gbf = get_post_meta( $funnel_id, 'is_global_funnel', true );
                        if( 'yes' !== $is_gbf ){
                            $offer_product = Wpfnl_Pro_functions::get_offer_product( $next_step['step_id'], $next_step['step_type'] );
                        }else{
                            if( is_plugin_active('wpfunnels-pro-gbf/wpfnl-pro-gb.php' )){
                                $offer_product = $this->skip_next_step_for_gbf( $funnel_id, $next_step['step_id'] );
                            }
                        }
                        $order_arg = array(
                            'customer' => $email,
                            'limit'    => -1
                        );
                        $orders = wc_get_orders($order_arg);
                        if( is_array($orders) ) {
                            $is_product_purchaged = false;
                            foreach( $orders as $customer_order ){
                                if( $is_product_purchaged ){
                                    break;
                                }
                                $order_date = $customer_order->get_date_created();
                                $date = $order_date->date('Y-m-d');
                                $current_date = date("Y-m-d");
                                $diff = date_diff(date_create($date),date_create($current_date));
                                $days = $diff->format("%R%a");
                                $days = str_replace('+','',$days);
                                foreach ($customer_order->get_items() as $item_id => $item) {
                                    $product = $item->get_product();
                                    if( $product ){
                                        $product_id = 'variation' == $product->get_type() ? $product->get_parent_id() : $product->get_id();
                                        if( $product_id ){
                                            if( 'yes' == $is_gbf ){
                                                $product = wc_get_product( isset($offer_product['id']) ? $offer_product['id'] : '' );
                                                if( $product && 'variation' == $product->get_type() ){
                                                    $offer_id =$product->get_parent_id();
                                                }elseif( $product ){
                                                    $offer_id =$product->get_id();
                                                }
                                                
                                                if( $product_id == $offer_id ){
                                                    if( $maybe_skip_within_days ){
                                                        if( $maybe_skip_within_days >= $days ){
                                                            $is_product_purchaged = true; 
                                                        }
                                                    }else{
                                                        $is_product_purchaged = true; 
                                                    }
                                                    break;
                                                }
                                            }else{
                                                $is_available = false;
                                                foreach( $offer_product as $_product ){
                                                    $product = wc_get_product( $_product['id'] );
                                                    if( $product && 'variation' == $product->get_type() ){
                                                        $offer_id =$product->get_parent_id();
                                                    }elseif( $product ){
                                                        $offer_id =$product->get_id();
                                                    }
                                
                                                    if( $product_id == $offer_id ){
                                                        if( $maybe_skip_within_days ){
                                                            if( $maybe_skip_within_days >= $days ){
                                                                $is_product_purchaged = true; 
                                                            }
                                                        }else{
                                                            $is_product_purchaged = true; 
                                                        }
                                                        break;
                                                    }
                                                }
                                                
                                                if( $is_product_purchaged ){
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                    
                                }
                            }
                            
                            if( $is_product_purchaged ){
                                $next_step = Wpfnl_functions::get_next_step( $funnel_id, $next_step['step_id'] );
                                return $this->modify_next_step_based_on_order( $next_step, $order );
                            }
                            
                        }
                    }
                }
            }
        }
        return $next_step;
    }

}
