<?php

namespace WPFunnelsPro\Frontend\Gateways;

use WPFunnels\Wpfnl_functions;

/**
 * Class Wpfnl_Pro_Mollie_Helper
 * @package WPFunnelsPro\Frontend\Gateways
 */
class Wpfnl_Pro_Mollie_Helper {


    /**
     * create customer id if user is not logged in
     *
     * @param $data
     * @param $order
     * @return mixed
     * @throws \Mollie\Api\Exceptions\ApiException
     */
    public function create_non_logged_in_mollie_customer_id( $data, $order ) {

        if ( isset( $data['payment']['customerId'] ) && null !== $data['payment']['customerId'] ) {
            return $data;
        }

        $checkout_id    = Wpfnl_functions::get_checkout_id_from_post_data();
        $funnel_id      = Wpfnl_functions::get_funnel_id_from_step($checkout_id);

        if( $checkout_id && $funnel_id ) {

            try {
                $billing_email      = $order->get_billing_email();
                $billing_first_name = $order->get_billing_first_name();
                $billing_last_name  = $order->get_billing_last_name();
                $customer_id        = $this->get_customer_id_by_email_from_prev_order( $billing_email );

                if ( null === $customer_id ) {

                    $user_name          = $billing_first_name . ' ' . $billing_last_name;
                    $mollie_settings    = \Mollie_WC_Plugin::getSettingsHelper();
                    $is_test_mode       = $mollie_settings->isTestModeEnabled();

                    // create mollie customer in that case
                    $customer = \Mollie_WC_Plugin::getApiHelper()->getApiClient( $is_test_mode )->customers->create(
                        array(
                            'name'     => trim( $user_name ),
                            'email'    => trim( $billing_email ),
                            'metadata' => array( 'order_id' => $order->get_id() ),
                        )
                    );
                    $customer_id = $customer->id;
                }

                $this->set_mollie_customer_id( $order, $customer_id );
                $data['payment']['customerId'] = $customer_id;
            } catch ( \Mollie\Api\Exceptions\ApiException $e ) {

            }

        }
        return $data;
    }


    /**
     * get customer id from previous order
     *
     * @param $email
     * @return mixed|null
     */
    public function get_customer_id_by_email_from_prev_order( $email ) {
        $mollie_customer_id = null;
        $prev_orders_query  = new \WP_Query(
            array(
                'post_type'   => 'shop_order',
                'meta_query'  => array(
                    'relation' => 'AND',
                    array(
                        'key'     => '_billing_email',
                        'value'   => $email,
                        'compare' => '=',
                    ),
                    array(
                        'key'     => '_mollie_customer_id',
                        'compare' => 'EXISTS',
                    ),
                ),
                'fields'      => 'ids',
                'order'       => 'ASC',
            )
        );

        $prev_orders = $prev_orders_query->get_posts();
        if( is_array($prev_orders) && !empty($prev_orders) ) {
            $prev_order_id      = $prev_orders[0];
            $prev_order         = wc_get_order( $prev_order_id );
            $mollie_customer_id = $prev_order->get_meta( '_mollie_customer_id', true );
        }

        return $mollie_customer_id;
    }


    /**
     * save customer id with order data
     *
     * @param $order
     * @param $customer_id
     */
    public function set_mollie_customer_id( $order, $customer_id ) {
        if( empty($customer_id) ) {
            return;
        }
        $order->update_meta_data( '_mollie_customer_id', $customer_id );
        $order->save();
    }


    /**
     * get mollie customer id from
     * order
     *
     * @param $order
     * @param $test_mode
     * @return string|null
     */
    public function get_mollie_user_id( $order, $test_mode ) {
        $user_id            = $order->get_customer_id();

        $mollie_customer_id = \Mollie_WC_Plugin::getDataHelper()->getUserMollieCustomerId( $user_id, $test_mode );
        if ( null === $mollie_customer_id ) {
            $mollie_customer_id = $order->get_meta( '_mollie_customer_id', true );
        }
        return $mollie_customer_id;
    }


    /**
     * save meta for refund request
     *
     * @param $parent_order
     * @param $child_order
     * @param $transaction_id
     */
    public function save_meta_for_refund( $parent_order, $child_order, $transaction_id ) {
        if ( ! empty( $transaction_id ) ) {
            $payment_mode = $parent_order->get_meta( '_mollie_payment_mode' );
            update_post_meta( $child_order->get_id(), '_mollie_payment_id', $transaction_id );
            update_post_meta( $child_order->get_id(), '_mollie_order_id', $transaction_id );
            update_post_meta( $child_order->get_id(), '_mollie_payment_mode', $payment_mode );
        }
    }


    /**
     * save meta for subscription products
     *
     * @param $subscription
     * @param $offer_product
     * @param $order
     */
    public function save_offer_subscription_meta( $subscription, $offer_product, $order ) {

        if( is_object($order) && 'mollie_wc_gateway_creditcard' === $order->get_payment_method() ) {
            $subscription_id = $subscription->get_id();
            update_post_meta( $subscription_id, '_mollie_customer_id', $order->get_meta( '_mollie_customer_id', true ) );
            update_post_meta( $subscription_id, '_mollie_payment_id', $order->get_meta( '_mollie_payment_id', true ) );
            update_post_meta( $subscription_id, '_mollie_payment_mode', $order->get_meta( '_mollie_payment_mode', true ) );
        }
    }


    /**
     * process refund offer
     *
     * @param $order
     * @param $data
     * @return bool
     * @throws \Mollie\Api\Exceptions\ApiException
     * @throws \Mollie\Api\Exceptions\IncompatiblePlatform
     */
    public function process_refund_offer( $order, $data ) {
        $transaction_id = $data['transaction_id'];
        $refund_amount  = number_format( $data['refund_amount'], 2 );
        $order_currency = $order->get_currency( $order );

        $request     = array();
        $response_id = false;

        if ( ! is_null( $refund_amount ) && isset( $transaction_id ) && class_exists( 'MollieApiClient' ) ) {

            $mollie = new \Mollie\Api\MollieApiClient();

            $settings  = \Mollie_WC_Plugin::getSettingsHelper();
            $test_mode = $settings->isTestModeEnabled();
            $api_key   = $settings->getApiKey( $test_mode );

            $mollie->setApiKey( $api_key );
            $payment = $mollie->payments->get( $transaction_id );

            if ( $payment->canBeRefunded() && $payment->amountRemaining->currency === $order_currency && $payment->amountRemaining->value >= $refund_amount ) {
                $refund      = $payment->refund(
                    array(
                        'amount' => array(
                            'currency' => $order_currency,
                            'value'    => $refund_amount,
                        ),
                    )
                );
                $response_id = $refund->id;
            }
        }
        return $response_id;
    }

}